@@ -106,6 +106,39 @@ class AttemptingToAddNonPoscBaseNode(PoscBaseException):
     """Attempted to add ad invalid object to child nodes."""
 
 
+class NotProvided:
+    """A value that is not provided. Used to indicate that a parameter is not provided."""
+    def __bool__(self):
+        return False
+
+    def __repr__(self):
+        return 'None'
+    
+    def __str__(self):
+        return ''
+
+NOT_PROVIDED = NotProvided()
+
+@dataclass
+class PoscGlobals:
+    """Global OpenSCAD parameters. Modifies the global parameters for the generated script."""
+    _fn: int | None = None
+    _fa: float | None = None
+    _fs: float | None = None
+    
+    def overrides(self, _fn: int | None, _fa: float | None, _fs: float | None) -> \
+        tuple[tuple[str, int | float | None], ...]:
+        return (
+            ('$fn', self._fn if _fn is None else _fn),
+            ('$fa', self._fa if _fa is None else _fa),
+            ('$fs', self._fs if _fs is None else _fs)
+        )
+
+# Setting the _fn, _fs and _fa variables will result in the header to
+# the generated script to include the global parameters.
+POSC_GLOBALS = PoscGlobals()
+
+
 class Arg(object):
     """Defines an argument and field for PythonOpenScad PoscBase based APIs."""
 
@@ -157,7 +190,7 @@ class Arg(object):
 
     def default_value_str(self):
         """Returns the default value as a string otherwise '' if no default provided."""
-        if self.default_value is None:
+        if self.default_value is None or self.default_value is NOT_PROVIDED:
             return ''
         try:
             return repr(self.default_value)
@@ -696,7 +729,7 @@ class PoscBase(PoscRendererBase):
             is_different_name = arg.name != arg.attr_name
             if is_different_name:
                 delattr(self, arg.name)
-            if value is not None:
+            if value is not None and value is not NOT_PROVIDED:
                 setattr(self, arg.attr_name, arg.typ(value))
             elif is_different_name:
                 setattr(self, arg.attr_name, None)
@@ -784,8 +817,12 @@ class PoscBase(PoscRendererBase):
     def get_modules(self):
         return ()
 
-    def dump_with_code_dumper(self, code_dumper: CodeDumper):
+    def dump_with_code_dumper(self, code_dumper: CodeDumper, _fn: int = None, _fa: float = None, _fs: float = None):
         """Returns the OpenScad equivalent code for this node."""
+        if code_dumper.DUMPS_OPENSCAD:
+            header = self._header(_fn, _fa, _fs)
+            if header:
+                code_dumper.add_line(header)
         code_dumper.add_modules(self.get_modules())
         self.code_dump(code_dumper)
         code_dumper.dump_modules()
@@ -815,26 +852,44 @@ class PoscBase(PoscRendererBase):
             if getattr(self, arg.attr_name) != getattr(other, arg.attr_name):
                 return False
         return self.children() == other.children()
+    
+    def _header(self, _fn: int = None, _fa: float = None, _fs: float = None) -> str:
+        header = ""
+        for name, val in POSC_GLOBALS.overrides(_fn, _fa, _fs):
+            if val is not None:
+                header += f"{name} = {val};\n"
+        return header
 
     # OpenPyScad compat functions.
-    def dumps(self):
+    def dumps(self, _fn: int = None, _fa: float = None, _fs: float = None) -> str:
         """Returns a string of this object's OpenScad script."""
-        return self.dump_with_code_dumper(CodeDumper()).writer.get()
+        return self.dump_with_code_dumper(CodeDumper(), _fn, _fa, _fs).writer.get()
 
-    def dump(self, fp):
+    def dump(self, fp, _fn: int = None, _fa: float = None, _fs: float = None):
         """Writes this object's OpenScad script to the given file.
         Args:
             fp: The python file object to use.
         """
-        self.dump_with_code_dumper(CodeDumper(writer=FileWriter(fp))).writer.finish()
+        self.dump_with_code_dumper(CodeDumper(writer=FileWriter(fp)), _fn, _fa, _fs).writer.finish()
 
-    def write(self, filename, encoding='utf-8'):
+    def write(self, filename, encoding='utf-8', _fn: int = None, _fa: float = None, _fs: float = None):
         """Writes the OpenScad script to the given file name.
         Args:
             filename: The filename to create.
         """
         with open(filename, 'w', encoding=encoding) as fp:
-            self.dump(fp)
+            self.dump(fp, _fn, _fa, _fs)
+            
+    def get_fn_fa_fs_args(self) -> dict[str, int | float]:
+        """Returns a dictionary of the fn, fa and fs arguments."""
+        args = {}
+        for attr in ['fn', 'fa', 'fs']:
+            if hasattr(self, attr):
+                val = getattr(self, attr)
+                if val is None:
+                    val = getattr(POSC_GLOBALS, attr)
+                args[attr] = val
+        return args
             
     def renderObj(self, renderer: M3dRenderer) -> RenderContext:
         assert False, "Not implemented"
@@ -920,11 +975,13 @@ class PoscBase(PoscRendererBase):
 
 # A decorator for PoscBase classes.
 def apply_posc_attributes(clazz):
-    """Decorator that applies an equivalent constructor with it\'s own generated
+    """Decorator that applies an equivalent constructor with it's own generated
     docstring. Also adds some SolidPython script compatibility class by providing an alias
-    class in the current module."""
+    class in the current module.
+    """
     if clazz.__init__ != PoscBase.__init__:
         raise InitializerNotAllowed('class %s should not define __init__' % clazz.__name__)
+    
     # Check for name collision.
     args: Tuple[Arg] = clazz.OSC_API_SPEC.args
     for arg in args:
@@ -938,6 +995,7 @@ def apply_posc_attributes(clazz):
     for arg in args:
         setattr(clazz, arg.name, arg.to_dataclass_field())
     dataclass(repr=False)(clazz)
+
     clazz.__init__.__doc__ = clazz.OSC_API_SPEC.generate_init_doc()
     strs = []
     if clazz.__doc__:
@@ -1087,7 +1145,7 @@ class Cylinder(PoscBase):
         'cylinder',
         (
             Arg('h', float, 1.0, 'height of the cylinder or cone.', required=True),
-            Arg('r', float, 1.0, 'radius of cylinder. r1 = r2 = r.'),
+            Arg('r', float, NOT_PROVIDED, 'radius of cylinder. r1 = r2 = r.'),
             Arg('r1', float, None, 'radius, bottom of cone.'),
             Arg('r2', float, None, 'radius, top of cone.'),
             Arg('d', float, None, 'diameter of cylinder. r1 = r2 = d / 2.'),
@@ -1134,6 +1192,22 @@ class Cylinder(PoscBase):
 
     def check_valid(self):
         """Checks that the values of cylinder satisfy OpenScad cylinder requirements."""
+        
+        if self.r is not NOT_PROVIDED and self.r1 is not None:
+            if self.r2 is None:
+                self.r2 = self.r1
+                self.r1 = self.r
+                self.r = None
+            elif self.r is not None:
+                raise TypeError("got multiple values for argument 'r2'")
+        
+        if all(x is None for x in [self.r1, self.r2, self.d1, self.d2, self.d]):
+            if self.r is NOT_PROVIDED:
+                self.r = 1.0
+        
+        if self.r is NOT_PROVIDED:
+            self.r = None
+        
         values = (('r1', self.get_r1()), ('r2', self.get_r2()))
         for k, v in values:
             if v is None:
@@ -1144,8 +1218,8 @@ class Cylinder(PoscBase):
         centre = True if self.center else False
         return renderer.cylinder(
             self, self.h, self.get_r1(), self.get_r2(), 
-            get_fragments_from_fn_fa_fs(self.get_r1(), self._fn, self._fa, self._fs),
-            centre)
+            get_fragments_from_fn_fa_fs(self.get_r1(), **self.get_fn_fa_fs_args()),
+            centre=centre)
 
 
 @apply_posc_attributes
@@ -1157,7 +1231,7 @@ class Sphere(PoscBase):
     OSC_API_SPEC = OpenScadApiSpecifier(
         'sphere',
         (
-            Arg('r', float, 1.0, 'radius of sphere. Ignores d if set.'),
+            Arg('r', float, NOT_PROVIDED, 'radius of sphere. Ignores d if set.'),
             Arg('d', float, None, 'diameter of sphere.'),
             FA_ARG,
             FS_ARG,
@@ -1175,14 +1249,23 @@ class Sphere(PoscBase):
 
     def check_valid(self):
         """Checks that the construction of cylinder is valid."""
-        if all(x is None for x in [self.r, self.d]):
+        
+        if self.d is not None:
+            if self.r is not NOT_PROVIDED and self.r is not None:
+                raise TypeError("got parameters r and d")
+            self.r = None
+            
+        elif self.r is NOT_PROVIDED:
+            self.r = 1.0
+        
+        if (self.r, self.d) == (None, None):
             raise RequiredParameterNotProvided(
                 'Both parameters r and d are None. A value for r or d must be provided.'
             )
         self.check_required_parameters()
     
     def renderObj(self, renderer: M3dRenderer) -> RenderContext:
-        return renderer.sphere(self, self.r, get_fragments_from_fn_fa_fs(self.r, self._fn, self._fa, self._fs))
+        return renderer.sphere(self, self.r, get_fragments_from_fn_fa_fs(self.r, **self.get_fn_fa_fs_args()))
 
 
 @apply_posc_attributes
@@ -1351,7 +1434,7 @@ class Offset(PoscParentBase):
         self.check_required_parameters()
         
     def renderObj(self, renderer: M3dRenderer) -> RenderContext:
-        return renderer.offset(self, self.r, self.delta, self.chamfer, self._fn, self._fa, self._fs)
+        return renderer.offset(self, self.r, self.delta, self.chamfer, **self.get_fn_fa_fs_args())
 
 
 @apply_posc_transformation_attributes
@@ -1461,7 +1544,7 @@ class Linear_Extrude(PoscParentBase):
     
     def renderObj(self, renderer: M3dRenderer) -> RenderContext:
         return renderer.linear_extrude(self, self.height, self.center, self.convexity, 
-                                       self.twist, self.slices, self.scale_, self._fn)
+                                       self.twist, self.slices, self.scale_, **self.get_fn_fa_fs_args())
 
 
 @apply_posc_transformation_attributes
@@ -1488,9 +1571,7 @@ class Rotate_Extrude(PoscParentBase):
         return renderer.rotate_extrude(self, 
                                        self.angle, 
                                        self.convexity, 
-                                       self._fn, 
-                                       self._fa, 
-                                       self._fs)
+                                       **self.get_fn_fa_fs_args())
 
 
 @apply_posc_attributes
@@ -1530,7 +1611,7 @@ class Circle(PoscBase):
             r = self.r
         else:
             r = self.d / 2  
-        fn = get_fragments_from_fn_fa_fs(r, self._fn, self._fa, self._fs)
+        fn = get_fragments_from_fn_fa_fs(r, **self.get_fn_fa_fs_args())
         return renderer.circle(self, r, fn)
 
 
@@ -1678,7 +1759,8 @@ class Polyhedron(PoscBase):
     )
 
     def renderObj(self, renderer: M3dRenderer) -> RenderContext:
-        return renderer.polyhedron(self, self.points, self.triangles, self.faces)
+        return renderer.polyhedron(
+            self, verts=self.points, faces=self.faces, triangles=self.triangles)
 
 @apply_posc_attributes
 class Union(PoscParentBase):
